<?php
/**
 * This is the entrypoint for the locrMAPS WordPress plugin.
 *
 * @package locrMAPS
 */

declare(strict_types=1);

namespace Locr\WordPress\Plugin\LocrMaps;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * A typed sequence (index-based array).
 *
 * @phpstan-implements \IteratorAggregate<int, mixed>
 */
class TypedSequence implements \IteratorAggregate, \Countable {
	/**
	 * The type.
	 *
	 * @var string
	 */
	protected string $type;
	/**
	 * The values.
	 *
	 * @var array<mixed>
	 */
	protected array $values = array();

	/**
	 * Constructor.
	 */
	protected function __construct() {
	}

	/**
	 * Adds a value to the sequence.
	 *
	 * @param mixed $value The new value to add.
	 * @throws \TypeError If the value is not of the correct type.
	 */
	public function add( $value ): self {
		if ( ! ( $value instanceof $this->type ) ) {
			throw new \TypeError( 'Invalid type' );
		}

		$this->values[] = $value;
		return $this;
	}

	/**
	 * Returns the amount of elements in the sequence.
	 */
	public function count(): int {
		return count( $this->values );
	}

	/**
	 * Returns a TypedSequence isntance for the given type.
	 *
	 * @param string $type The type.
	 */
	public static function for_type( string $type ): self {
		$ret       = new self();
		$ret->type = $type;
		return $ret;
	}

	/**
	 * Gets an ArrayIterator for this sequence.
	 *
	 * @return \ArrayIterator<int, mixed>
	 */
	public function getIterator(): \ArrayIterator {
		return new \ArrayIterator( $this->values );
	}

	/**
	 * Gets an ordered ArrayIterator for this sequence.
	 *
	 * @param callable $compare The comparison function.
	 * @return \ArrayIterator<int, mixed>
	 */
	public function getOrderedIterator( callable $compare ): \ArrayIterator {
		$values = array();
		foreach ( $this->values as $value ) {
			$values[] = $value;
		}
		usort( $values, $compare );
		return new \ArrayIterator( $values );
	}

	/**
	 * Converts the sequence to an array.
	 *
	 * @return array<mixed>
	 */
	public function to_array(): array {
		return iterator_to_array( $this, false );
	}
}
