/* global L */

export default class Icon {
	#id = 0;
	#url = '';
	#size = {
		width: 0,
		height: 0,
	};
	#name = '';
	#scale = 1.0;
	#anchor = 'center';
	#draggable = false;

	/**
	 * @param {string|Object|null} data
	 */
	constructor( data ) {
		if ( typeof data === 'string' && data !== '' ) {
			try {
				const json = JSON.parse( data );
				this.#parseObject( json );
			} catch ( exc ) {
				throw new Error( 'Icon: Failed to parse string as JSON.' );
			}
		} else if ( data !== null && typeof data === 'object' ) {
			this.#parseObject( data );
		} else {
			throw new Error(
				`Icon: Invalid type (${ typeof data }) of data parameter.`
			);
		}
	}

	get anchor() {
		return this.#anchor;
	}

	get draggable() {
		return this.#draggable;
	}

	set draggable( value ) {
		if ( typeof value !== 'boolean' ) {
			throw new Error( 'Icon: "draggable" must be a boolean.' );
		}

		this.#draggable = value;
	}

	get id() {
		return this.#id;
	}

	get name() {
		return this.#name;
	}

	get url() {
		return this.#url;
	}

	get scale() {
		return this.#scale;
	}

	get size() {
		return this.#size;
	}

	/**
	 * @param {Object} obj
	 */
	#parseObject( obj ) {
		this.#parseObjectPropertyId( obj );
		this.#parseObjectPropertyUrl( obj );
		this.#parseObjectPropertySize( obj );
		this.#parseObjectPropertyName( obj );
		this.#parseObjectPropertyScale( obj );
		this.#parseObjectPropertyAnchor( obj );
		this.#parseObjectPropertyDraggable( obj );
	}

	/**
	 * @param {Object} obj
	 */
	#parseObjectPropertyAnchor( obj ) {
		if ( obj.hasOwnProperty( 'anchor' ) ) {
			if ( typeof obj.anchor !== 'string' ) {
				throw new Error( 'Icon: "anchor" must be a string.' );
			}

			switch ( obj.anchor ) {
				case 'center':
				case 'bottom':
					this.#anchor = obj.anchor;
					break;

				default:
					throw new Error(
						'Icon: "anchor" must be either "center" or "bottom".'
					);
			}
		}
	}

	/**
	 * @param {Object} obj
	 */
	#parseObjectPropertyDraggable( obj ) {
		if ( obj.hasOwnProperty( 'draggable' ) ) {
			if ( typeof obj.draggable !== 'boolean' ) {
				throw new Error( 'Icon: "draggable" must be a boolean.' );
			}

			this.#draggable = obj.draggable;
		}
	}

	/**
	 * @param {Object} obj
	 */
	#parseObjectPropertyId( obj ) {
		if ( ! obj.hasOwnProperty( 'id' ) ) {
			throw new Error( 'Icon: Missing "id" in JSON-object.' );
		}
		if ( typeof obj.id !== 'number' ) {
			throw new Error( 'Icon: "id" must be a number.' );
		}
		if ( obj.id < 0 ) {
			throw new Error( 'Icon: "id" must be a positive number.' );
		}
		this.#id = obj.id;
	}

	/**
	 * @param {Object} obj
	 */
	#parseObjectPropertyName( obj ) {
		if ( obj.hasOwnProperty( 'name' ) ) {
			if ( typeof obj.name !== 'string' ) {
				throw new Error( 'Icon: "name" must be a string.' );
			}
			this.#name = obj.name.trim();
		}
	}

	/**
	 * @param {Object} obj
	 */
	#parseObjectPropertyScale( obj ) {
		if ( obj.hasOwnProperty( 'scale' ) ) {
			if ( typeof obj.scale !== 'number' ) {
				throw new Error( 'Icon: "scale" must be a number.' );
			}
			if ( obj.scale <= 0 ) {
				throw new Error( 'Icon: "scale" must be a positive number.' );
			}
			if ( obj.scale > 10.0 ) {
				throw new Error(
					'Icon: "scale" must be less than or equal to 10.0.'
				);
			}
			this.#scale = obj.scale;
		}
	}

	/**
	 * @param {Object} obj
	 */
	#parseObjectPropertySize( obj ) {
		if ( ! obj.hasOwnProperty( 'size' ) ) {
			throw new Error( 'Icon: Missing "size" in JSON-object.' );
		}
		if ( ! obj.size.hasOwnProperty( 'width' ) ) {
			throw new Error( 'Icon: Missing "size.width" in JSON-object.' );
		}
		if ( ! obj.size.hasOwnProperty( 'height' ) ) {
			throw new Error( 'Icon: Missing "size.height" in JSON-object.' );
		}

		if ( typeof obj.size.width !== 'number' ) {
			throw new Error( 'Icon: "size.width" must be a number.' );
		}
		if ( obj.size.width <= 0 ) {
			throw new Error( 'Icon: "size.width" must be a positive number.' );
		}

		if ( typeof obj.size.height !== 'number' ) {
			throw new Error( 'Icon: "size.height" must be a number.' );
		}
		if ( obj.size.height <= 0 ) {
			throw new Error( 'Icon: "size.height" must be a positive number.' );
		}

		this.#size.width = obj.size.width;
		this.#size.height = obj.size.height;
	}

	/**
	 * @param {Object} obj
	 */
	#parseObjectPropertyUrl( obj ) {
		if ( ! obj.hasOwnProperty( 'url' ) ) {
			throw new Error( 'Icon: Missing "url" in JSON-object.' );
		}
		if ( typeof obj.url !== 'string' ) {
			throw new Error( 'Icon: "url" must be a string.' );
		}
		if ( obj.url.trim() === '' ) {
			throw new Error( 'Icon: "url" must not be an empty string.' );
		}
		this.#url = obj.url.trim();
	}

	getDisplaySize() {
		return {
			width: this.#size.width * this.#scale,
			height: this.#size.height * this.#scale,
		};
	}

	/**
	 * @return {L.Icon} Leaflet icon
	 */
	toLeafletIcon() {
		return L.icon( this.toLeafletOptions() );
	}

	/**
	 * @return {{iconUrl: string, iconScale: number, iconSize: number[], iconAnchor: number[]}} Leaflet options
	 */
	toLeafletOptions() {
		const displaySize = this.getDisplaySize();

		const options = {
			iconUrl: this.#url,
			iconScale: this.#scale,
			iconSize: [ displaySize.width, displaySize.height ],
			iconAnchor: [ displaySize.width / 2, displaySize.height / 2 ],
		};

		if ( this.#anchor === 'bottom' ) {
			options.iconAnchor = [ displaySize.width / 2, displaySize.height ];
		}

		return options;
	}
}
